from .....com.orbitgis.tool.message.Message import Message
from .AMap import AMap
from .SDKViewer3DM import SDKViewer3DM
from .SerializeUtil import SerializeUtil
from .Viewer3DM import Viewer3DM
from .SDKWebSocketServer import SDKWebSocketServer

class SDKViewerComponent3DM:

    # Create an instance of the 3DMapping Viewer
    # Call the "initializeViewer" method before using any of the other viewer methods
    #
    def __init__(self):
        Message.print(__name__,"creation")

    # Initialize the 3D Mapping viewer.
    # Parameter applicationName: the name of the application that is using this viewer
    # Parameter options: a set of parameters to create the javascript viewer with
    # Parameter webSocketPort: the port number to bind the websocket server to
    #
    def initializeViewer(self,applicationName: str,options: AMap[object],webSocketPort: int) -> None:
        Message.print(__name__,"initializeViewer (websocket)")
        #
        Message.print(__name__,"creating websocket server")
        self.webSocketServer = SDKWebSocketServer()
        Message.print(__name__,"creating viewer")
        self.viewer = SDKViewer3DM(self.webSocketServer,applicationName,options)
        self.webSocketServer.setCommandHandler(self.viewer.processWebSocketCommand)
        #
        Message.print(__name__,"starting websocket server")
        self.webSocketServer.startServerThread("localhost",webSocketPort)
        Message.print(__name__,"started websocket server")

    # Get the 3D Mapping viewer.
    # Returns: the viewer (null if the viewer has not been initialized)
    #
    def getSDKViewer(self) -> SDKViewer3DM:
        return self.viewer
        
    # Get the 3D Mapping Viewer
    # Returns: the viewer (null if the viewer has not been initialized)
    #
    def getViewer(self) -> Viewer3DM:
        return self.viewer.getBridgeViewer()
