# NOTE: this class has been translated from Java.
# Do not modify this file, changes will be overwritten.

#package orbitgt.mapping3d.sdk.viewer;

from abc import ABC

from .....orbitgt.mapping3d.sdk.viewer.AList import AList
from .....orbitgt.mapping3d.sdk.viewer.AMap import AMap
from .....orbitgt.mapping3d.sdk.viewer.Bounds import Bounds
from .....orbitgt.mapping3d.sdk.viewer.Collections import Collections
from .....orbitgt.mapping3d.sdk.viewer.Constants import Constants
from .....orbitgt.mapping3d.sdk.viewer.Coordinate import Coordinate
from .....orbitgt.mapping3d.sdk.viewer.MapObject import MapObject
from .....orbitgt.mapping3d.sdk.viewer.MeasureState import MeasureState
from .....orbitgt.mapping3d.sdk.viewer.Promise import Promise
from .....orbitgt.mapping3d.sdk.viewer.Signal import Signal
from .....orbitgt.mapping3d.sdk.viewer.SerializeUtil import SerializeUtil
from .....orbitgt.mapping3d.sdk.viewer.ViewerActions import ViewerActions

#
# Class Viewer3DM defines the main interface to the 3DMapping Viewer SDK. See
# the developer documentation at <https://orbitgt.com/kb/3dmapping_sdk>
#


class Viewer3DM(ABC):
    #
    # Promise that resolves when all javascript code has been loaded and the viewer is ready for use. <br/>
    # Wait for this promise to resolve before calling any other methods.
    # Polling is possible by calling the `isResolved()` method (with boolean result) of the promise.
    #
    #public Promise['Viewer3DM'] isReady

    #
    # Signal that is dispatched after the application state has changed. <br/>
    # The new state is passed as an argument. Save this state to disk and later pass it as
    # the `Constants.STARTUP_APP_STATE` option in the startup options of the viewer
    # to enable auto-login at the next start.
    #
    #public Signal[str] onAppStateChanged
    #
    # Signal that is dispatched after the footprints have changed. <br/>
    # A optional message is passed as an argument. Call the `getFootprints2D()` method to
    # fetch new map objects to display on the host reference map.
    #
    #public Signal[str] onFootprintsChanged
    #
    # Signal that is dispatched after the field-of-views have changed. <br/>
    # A optional message is passed as an argument. Call the `getFieldOfViews2D()` method to
    # fetch new map objects to display on the host reference map.
    #
    #public Signal[str] onFieldOfViewsChanged
    #
    # Signal that is dispatched after the globals have changed. <br/>
    # A optional message is passed as an argument. Call the `getGlobals()` method to fetch new map
    # objects to display on the host reference map.
    #
    #public Signal[str] onGlobalsChanged
    #
    # Signal that is dispatched after the measurement has changed. <br/>
    # A optional message is passed as an argument. Call the `getMeasureState()` method to
    # fetch new map objects (inside the measure state) to display on the host
    # reference map.
    #
    #public Signal[str] onMeasureStateChanged
    #
    # Signal that is dispatched after the user requested an overview to the extent of a resource. <br/>
    # The spatial extent of that resource is passed as an argument.
    # Zoom the host reference map to the given spatial extent.
    #
    #public Signal[Bounds] onZoomToExtent
    #
    # Signal that is dispatched after the focus position has been changed. <br/>
    # The new spatial position is passed as an argument (can be null if the focus has been
    # cleared). Center (move) the host reference map to this position if it falls
    # outside the current visible extent of the host reference map.
    #
    #public Signal[Coordinate] onFocusPositionChanged
    #
    # Signal that is dispatched when the viewer needs a location to be picked on the host reference map by the user.<br/>
    # A optional message is passed as an argument. When a location has been picked by the user, use the
    # `setPickPosition()` method to pass it to the viewer.
    #
    #public Signal[str] onPickPositionRequested
    #
    # Signal that is dispatched when the viewer does not need the user to pick a location anymore. <br/>
    # A optional message is passed as an argument. Cancel any
    # active pick function and return the host reference map to its default (pan)
    # function.
    #
    #public Signal[str] onPickPositionCancelled
    #
    # Signal that is dispatched when the viewer has edited an existing overlay map object.<br/>
    #  The updated map object is passed as an argument.
    #
    #public Signal[MapObject] onOverlayObjectChanged
    #
    # Signal that is dispatched when the viewer has deleted an existing overlay map object.<br/>
    # The deleted map object is passed as an argument.
    #
    #public Signal[MapObject] onOverlayObjectDeleted
    #
    # Signal that is dispatched when the viewer has created a new overlay map object.<br/>
    # The new map object is passed as an argument and it only contains
    # geometry, in the `spatialPoint` or `spatialLine` or `spatialArea` property of
    # the map object. Make sure to call the `addOverlayObjects` method after
    # initializing the `objectId` and graphic properties like color.
    #
    #public Signal[MapObject] onOverlayObjectCreated

    #
    # Signal that is dispatched when the workspace (and all resources in it) are open and ready for use.<br/>
    # Wait for this signal if you want to invoke actions.
    #
    #public Signal[str] onWorkspaceOpened

    #
    # Signal that is disaptched when the workspace is closed.
    #
    #public Signal[str] onWorkspaceClosed

    #
    # Create a new instance of the viewer.
    #
    # @param applicationName the name of the application that is using the SDK.
    # @param options         the map of startup options.
    #
    def __init__(self,applicationName: str,options: AMap[object]):
        self.isReady = None
        self.onAppStateChanged = Signal[str]()
        self.onFootprintsChanged = Signal[str]()
        self.onFieldOfViewsChanged = Signal[str]()
        self.onGlobalsChanged = Signal[str]()
        self.onMeasureStateChanged = Signal[str]()
        self.onZoomToExtent = Signal[Bounds]()
        self.onFocusPositionChanged = Signal[Coordinate]()
        self.onPickPositionRequested = Signal[str]()
        self.onPickPositionCancelled = Signal[str]()
        self.onOverlayObjectChanged = Signal[MapObject]()
        self.onOverlayObjectDeleted = Signal[MapObject]()
        self.onOverlayObjectCreated = Signal[MapObject]()
        self.onWorkspaceOpened = Signal[str]()
        self.onWorkspaceClosed = Signal[str]()

    #
    # Update the size of the viewer when the host window has resized.
    #
    # @param width  The new width (pixels).
    # @param height The new height (pixels).
    #
    def setSize(self,width: float,height: float) -> None:
        pass


    #
    # Get the footprint map objects to display on the 2D host reference map. <br/>
    # This method should be called after the `onFootprintsChanged` signal has been received.
    #
    # @param viewbounds       the spatial bounds of the 2D host reference map.
    # @param scaleDenominator the scale denominator of the 2D host reference map
    #                         (like 1500.0 for example).
    # @return Returns a promise to the footprint objects.
    #
    def getFootprints2D(self,viewbounds: Bounds,scaleDenominator: float) -> Promise[AList[MapObject]]:
        pass


    #
    # Get the field-of-view map objects to display on the host reference map. <br/>
    # This method should be called after the `onFieldOfViewsChanged` signal has been received.
    #
    # @return Returns the field-of-view map objects.
    #
    def getFieldOfViews2D(self) -> AList[MapObject]:
        pass


    #
    # Get the global map objects to display on the host reference map. This method
    # should be called after the `onGlobalsChanged` signal has been received.
    #
    # @return Returns the global map objects.
    #
    def getGlobals(self) -> AList[MapObject]:
        pass


    #
    # Get the state of the measurement and the map objects to display it on the
    # host reference map. This method should be called after the
    # `onMeasureStateChanged` signal has been received.
    #
    # @return Returns the current measure state (with embedded map objects) or null
    #         in case the measure function is not active.
    #
    def getMeasureState(self) -> MeasureState:
        pass


    #
    # Get the current focus position of the viewer.
    #
    # @return Returns the current focus position (can be null).
    #
    def getFocusPosition(self) -> Coordinate:
        pass


    #
    # Set the focus position of the viewer. Every view will bring this position into view (opening the closest or best photo as needed).
    #
    # @param position The position to focus on (either 2D or 3D).
    #
    def setFocusPosition(self,position: Coordinate) -> None:
        pass


    #
    # Communicate a position that was picked by the user on the host reference map
    # to the viewer. Calling this function only has effect when it is called after
    # the `onPickPositionRequested` signal has been received.
    #
    # @param position The user-chosen position on the host reference map.
    #
    def setPickPosition(self,position: Coordinate) -> None:
        pass


    #
    # Add a new view.
    #
    # @param position           The position to focus on (either 2D or 3D).
    # @param renderMode         The render mode of the view to open. Have a look at
    #                           the various `RENDER_MODE_` values in the
    #                           `Constants` class. Use `RENDER_MODE_3D` or
    #                           `RENDER_MODE_PHOTO_SPHERICAL` for example.
    # @param closeExistingViews Should all existing views be closed?
    # @return returns the promise of a status message. See the various
    #         `ADD_VIEW_ERROR_` values in the `Constants` class.
    #
    def addView(self,position: Coordinate,renderMode: str,closeExistingViews: bool) -> Promise[str]:
        pass


    #
    # Adds a list of objects as overlays on the viewer. A resource for holding
    # these map objects will be added to the workspace of the viewer if it does not
    # exist yet.
    #
    # @param list a list of objects to add.
    #
    def addOverlayObjects(self,list: AList[MapObject]) -> None:
        pass


    #
    # Clear the list of overlay objects from the viewer.
    #
    def clearOverlayObjects(self) -> None:
        pass


    #
    # Transform a coordinate to a different CRS.
    #
    # @param coordinate The source coordinate to transform.
    # @param targetCRS  The target EPSG code.
    # @return Coordinate The coordinate expressed in the target CRS.
    #
    def transformCoordinate(self,coordinate: Coordinate,targetCRS: str) -> Coordinate:
        pass


    #
    # Get a property. Note: This function is only for internal SDK use, application
    # builders do not have to call it.
    #
    # @param propertyName The name of the property.
    # @return Returns the property value.
    #
    def getProperty(self,propertyName: str) -> object:
        pass


    #
    # Set a property. Note: This function is only for internal SDK use, application
    # builders do not have to call it.
    #
    # @param propertyName  The name of the property.
    # @param propertyValue The value of the property.
    #
    def setProperty(self,propertyName: str,propertyValue: object) -> None:
        pass


    #
    # Perform an action. Note: This function is only for internal SDK use,
    # application builders do not have to call it.
    #
    # Some actions are documented in the `ViewerActions` class.
    #
    # @param action       The action to perform.
    # @param argumentList The arguments.
    # @return Returns the result.
    #
    def doAction(self,action: str,argumentList: AList[object]) -> object:
        pass

